/*
 *  Name        : kclock.cpp
 *  Version     : 0.6
 *  Description : A clock with some more...
 *
 *  Copyright (C) 1998 Luca Montecchiani <m.luca@usa.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "kclock.h"
#include "kclock.moc"

Kclock::Kclock( QWidget *parent, const char *name )
	: QWidget( parent, name )
{
	initMetaObject();

	// start internal timer
	QTimer *internalTimer = new QTimer( this );
	connect( internalTimer, SIGNAL(timeout()), SLOT(Timeout()) );
	// internalTimer->start( 1000 );
	internalTimer->start( 100 );

	// define the menu
	menu      = new QPopupMenu();
	menu1     = new QPopupMenu();
	menu2     = new QPopupMenu();
	dateID    = menu->insertItem(klocale->translate("Show date"), this, SLOT(Show_date()));
	secondID  = menu->insertItem(klocale->translate("Show seconds"), this, SLOT(Show_second()));
	ontopID   = menu->insertItem(klocale->translate("Always on top"), this, SLOT(Show_ontop()));
	digitalID = menu->insertItem(klocale->translate("Digital display"), this, SLOT(Show_digital()));
	beepID    = menu->insertItem(klocale->translate("Beep every hour"), this, SLOT(Beep()));
	speakID   = menu->insertItem(klocale->translate("Speak every hour"), this, SLOT(Speak()));
	tree_dID  = menu->insertItem(klocale->translate("3D effect"), this, SLOT(Show_tree_d()));
	borderID  = menu->insertItem(klocale->translate("Show border"), this, SLOT(Show_border()));

	menu->insertItem(klocale->translate("Font..."), menu1);
	menu1->insertItem(klocale->translate("Digital Font"), this, SLOT(selectFont()));
	menu1->insertSeparator();
	menu1->insertItem(klocale->translate("Reset Font"), this, SLOT(ResetFont()));

	menu->insertItem(klocale->translate("Colors..."), menu2);
	menu2->insertItem(klocale->translate("Background"), this, SLOT(selectBack()));
	menu2->insertItem(klocale->translate("Foreground"), this, SLOT(selectFore()));
	menu2->insertItem(klocale->translate("Shadow"), this, SLOT(selectShad()));
	menu2->insertSeparator();
	menu2->insertItem(klocale->translate("Reset Colors"), this, SLOT(resetCol()));

	menu->insertSeparator();
	menu->insertItem(klocale->translate("Set system Date/Time"), this, SLOT(Set_time()));
	menu->insertSeparator();
	menu->insertItem(klocale->translate("Save setting"), this, SLOT(Save_config()));
	menu->insertItem(klocale->translate("Help"), this, SLOT(Help()));
	menu->insertItem(klocale->translate("About"), this, SLOT(About()));
	menu->insertSeparator();
	menu->insertItem(klocale->translate("Exit"), this, SLOT(Quit()));
	menu->setCheckable( TRUE );

	// reading rc file
	onmenu = FALSE;

	KConfig	*config = kapp->getConfig();

	config->setGroup("Geometry");
	setGeometry( config->readNumEntry("TopLevelGeometry-x", 0), config->readNumEntry("TopLevelGeometry-y", 0), config->readNumEntry("TopLevelGeometry-w", 100), config->readNumEntry("TopLevelGeometry-h", 100) );

	config->setGroup("Option");
	date 	= !config->readNumEntry("date", FALSE);
	second 	= !config->readNumEntry("second", TRUE);
	digital = !config->readNumEntry("digital", FALSE);
	ontop 	= !config->readNumEntry("ontop", FALSE);
	beep 	= !config->readNumEntry("beep", FALSE);
	speak 	= !config->readNumEntry("speak", FALSE);
	tree_d 	= !config->readNumEntry("tree_d", TRUE);
	border 	= !config->readNumEntry("border", TRUE);
	ver 	= config->readNumEntry("ver", 0);
	config->writeEntry("ver", VER);

	config->setGroup( "Font" );
	font.setFamily(config->readEntry( "Family", "Utopia"));
	font.setPointSize(config->readNumEntry( "Point Size", 51));
	font.setWeight(config->readNumEntry( "Weight", 75));
	font.setItalic(config->readNumEntry( "Italic",TRUE ));
	font.setBold(config->readNumEntry( "Bold",TRUE ));

	config->setGroup( "Color" );
	QColor colortmp = lightGray;
	Back = config->readColorEntry( "Background" , &colortmp );
	colortmp = black;
	Fore = config->readColorEntry( "Foreground" , &colortmp );
	colortmp = gray;
	Shad = config->readColorEntry( "Shadow" , &colortmp );

	setBackgroundColor(Back);

	if ( ver != VER ) Hints();

	// settings up some things
	Show_ontop();
	Show_digital();
	Show_second();
	Show_date();
	Beep();
	Speak();
	Show_tree_d();
	Show_border();

	sound = FALSE;
}


void Kclock::resetCol()
{
	Back = lightGray;
	Fore = black;
	Shad = gray;
	setBackgroundColor(Back);
	Initialize();
}

void Kclock::selectBack()
{
	onmenu = TRUE;
	KColorDialog::getColor(Back);
	setBackgroundColor(Back);
	onmenu = FALSE;
	Initialize();
}

void Kclock::selectFore()
{
	onmenu = TRUE;
	KColorDialog::getColor(Fore);
	onmenu = FALSE;
	Initialize();
}

void Kclock::selectShad()
{
	onmenu = TRUE;
	KColorDialog::getColor(Shad);
	onmenu = FALSE;
	Initialize();
}

void Kclock::selectFont()
{
	onmenu = TRUE;
	KFontDialog::getFont(font);
	onmenu = FALSE;
}

void Kclock::ResetFont()
{
	onmenu = TRUE;
	font.setFamily("Utopia");
	font.setPointSize(51);
	font.setWeight(75);
	font.setItalic(TRUE);
	font.setBold(TRUE);     
	onmenu = FALSE;
}

void Kclock::Save_config()
{
	KConfig *config = kapp->getConfig();

	config->setGroup("Option");
	config->writeEntry("second",second);
	config->writeEntry("date",date);
	config->writeEntry("digital",digital);
	config->writeEntry("ontop",ontop);
	config->writeEntry("beep",beep);
	config->writeEntry("speak",speak);
	config->writeEntry("tree_d",tree_d);
	config->writeEntry("border",border);

	config->setGroup("Geometry");
	config->writeEntry("TopLevelGeometry-x",x());
	config->writeEntry("TopLevelGeometry-y",y());
	config->writeEntry("TopLevelGeometry-w",width());
	config->writeEntry("TopLevelGeometry-h",height());

	config->setGroup( "Font" );
	config->writeEntry( "Family",font.family());
	config->writeEntry( "Point Size",font.pointSize());
	config->writeEntry( "Weight",font.weight());
	config->writeEntry( "Italic",font.italic());
	config->writeEntry( "Bold",font.bold());

	config->setGroup( "Color" );
	config->writeEntry( "Background" , Back );
	config->writeEntry( "Foreground" , Fore );
	config->writeEntry( "Shadow" , Shad );
	
	config->sync();
}

void Kclock::Show_tree_d()
{
	tree_d = !tree_d;
	menu->setItemChecked( tree_dID, tree_d );
}

void Kclock::Show_second()
{
	second = !second;
	menu->setItemChecked( secondID, second );
}

void Kclock::Show_ontop()
{
	ontop = !ontop;
	menu->setItemChecked( ontopID, ontop );
	setFocusPolicy( QWidget::StrongFocus );
}

void Kclock::Show_digital()
{
	digital = !digital;
	menu->setItemChecked( digitalID, digital );
	if ( !digital )
		{
		if ( height() < width() )
			resize( height(),height());
		else	resize( width() ,width() );
		}
}

void Kclock::Speak()
{
	speak = !speak;
	menu->setItemChecked( speakID, speak );
}

void Kclock::Beep()
{
	beep = !beep;
	menu->setItemChecked( beepID, beep );
}

void Kclock::Quit()
{
	delete menu;
	delete menu1;
	delete menu2;
	kapp->quit();   
}

void Kclock::Set_time()
{
	onmenu = TRUE;
	Buf=kapp->kde_bindir()+"/kcmclock";
	int res = system(Buf);
	if ( res != 0 )
		{
		Buf.sprintf("Kclock error #%d",res/256);
		QMessageBox::warning( 0, Buf,klocale->translate("setting new Date/Time !!"));
		}
	if ( date )
		{
		QDate d = QDate::currentDate();
		setCaption(d.toString());
		}
	onmenu = FALSE;
}

void Kclock::Timeout()
{
	// get current time
	time = QTime::currentTime();

	// Multimedia Stuff
	if ( time.minute() == 0 )
		{
		if ( !sound )
			{
			sound = TRUE;
			if ( beep )
				kapp->beep();
			if ( speak )
				{
				KAudio KAServer;
				KAServer.setAutosync(TRUE);

				Buf=kapp->kde_datadir()+"/kclock/sounds/thetimeis.wav";
				KAServer.play(Buf); 

				Buf.sprintf("/kclock/sounds/%02d.wav",time.hour());
				Buf=kapp->kde_datadir()+Buf;
				KAServer.play(Buf); 

				Buf=kapp->kde_datadir()+"/kclock/sounds/oclock.wav";
				KAServer.play(Buf); 
				}
			}
		}
	else	sound = FALSE;

	// check if saver is running !!!
	if ( (ontop) && (!hasFocus()) && (!menu->isVisible()) && (!onmenu) ) raise();

	// force repain for kclock area
	repaint();
}

void Kclock::Help()
{
	kapp->invokeHTMLHelp("kclock/kclock.html","");
}                                                        

void Kclock::Hints()
{
	onmenu = TRUE;
	QMessageBox::about( 0, "Welcome to kclock 0.6 !", 
	klocale->translate( "Mouse buttons actions:\n\n"\
	"Left : move the window when kclock is in no-border mode\n\n"\
	"Mid : copy Date/Time on the clipboard\n\n"\
	"Right : show kclock menu"));
	onmenu = FALSE;
}

void Kclock::About()
{
	onmenu = TRUE;
	QMessageBox::about( 0, "About kclock 0.6",
	"Kclock 0.6\n\n"\
	"(C) 1998 Luca Montecchiani (m.luca@usa.net)\n\n"\
	"http://www.geocities.com/SiliconValley/Vista/2964/");
	onmenu = FALSE;
}

void Kclock::Initialize()
{
	// Get icon !!
	QPixmap p1( kclock_xpm );
	setIcon(p1);

	// flicker free code by Remi Guyomarch <rguyom@mail.dotcom.fr>
	setBackgroundMode( NoBackground );

	// very sensible !!
	setFocusPolicy( QWidget::StrongFocus );
}

void Kclock::Show_border()
{
	border = !border;
	menu->setItemChecked( borderID, border );
	if ( border )
		recreate(parentWidget(),0,pos(),TRUE);
	else 	recreate(parentWidget(),WStyle_Customize | WState_ClickToFocus, pos(), TRUE);
	//else 	recreate(parentWidget(),WStyle_Customize | WStyle_NoBorder | WState_ClickToFocus, pos(), TRUE);
	Initialize();
}

void Kclock::Show_date()
{
	date = !date;
	menu->setItemChecked( dateID, date );
	if ( date )
		{
		QDate d = QDate::currentDate();
		setCaption(d.toString());
		}
	else	setCaption("kclock");
}

void Kclock::mousePressEvent(QMouseEvent *event)
{
	if ( event->button() == LeftButton )
		raise();
	if ( event->button() == RightButton )
		menu->popup(mapToGlobal(event->pos()));
	if ( event->button() == MidButton )
		{
		Buf = QDate::currentDate().toString() + "\n" + QTime::currentTime().toString();
		QClipboard *cb = QApplication::clipboard();
		cb->setText( Buf );
		QMessageBox::information( 0, "kclock",klocale->translate("Date/Time copied on the clipboard."));
		}
}

void Kclock::mouseMoveEvent( QMouseEvent *event )
{
	move(mapToGlobal(event->pos()));
}

void Kclock::resizeEvent( QResizeEvent *e )
{
	if ( digital )
		{
		// dc->resize(e->size());
		return;
		}
	if ( e->size().height() < e->size().width() )
		resize( e->size().height(),e->size().height());
	else	resize( e->size().width() ,e->size().width() );
}

void Kclock::paintEvent( QPaintEvent * )
{
	if ( !isVisible() )
		return;

	// flicker free code by Remi Guyomarch <rguyom@mail.dotcom.fr>
	QPixmap pm( size() );
	QPainter paint;
	pm.fill( Back );
	paint.begin( &pm );

	if ( !border )
		paint.drawRect(0,0,width(),height());

	if ( digital )
		{
		if ( second )
			Buf.sprintf("%02d:%02d:%02d",time.hour(),time.minute(),time.second());
		else	Buf.sprintf("%02d:%02d",time.hour(),time.minute());
		font.setPointSize(QMIN((int)(width()/Buf.length()*1.5),height()));
		paint.setFont( font );
		if ( tree_d )
			{
			paint.setPen( Shad );
			paint.drawText(3,3,width(),height(),AlignHCenter|AlignVCenter,Buf,-1,0,0);
			}
		paint.setPen( Fore );
		paint.drawText(rect(),AlignHCenter|AlignVCenter,Buf,-1,0,0);
		}
	else 	{
		QPointArray pts;
		QPoint cp = rect().center();
		int d = QMIN(width(),height());
		if ( tree_d )
			{
			paint.setPen( Shad );
			paint.setBrush( Shad );
			paint.setViewport(4,4,width(),height());
			}
		else 	{
			paint.setPen( Fore );
			paint.setBrush( Fore );
			}

		for ( int c=0 ; c < 2 ; c++ )
			{
			QWMatrix matrix;
			matrix.translate( cp.x(), cp.y() );
			matrix.scale( d/1000.0F, d/1000.0F );

			// lancetta delle ore 
			float h_angle = 30*(time.hour()%12-3) + time.minute()/2;
			matrix.rotate( h_angle );
			paint.setWorldMatrix( matrix );
			pts.setPoints( 4, -20,0,  0,-20, 300,0, 0,20 );
			paint.drawPolygon( pts );
			matrix.rotate( -h_angle );

			// lancetta dei minuti
			float m_angle = (time.minute()-15)*6;
			matrix.rotate( m_angle );
			paint.setWorldMatrix( matrix );
			pts.setPoints( 4, -10,0, 0,-10, 400,0, 0,10 );
			paint.drawPolygon( pts );
			matrix.rotate( -m_angle );

			// lancetta dei secondi
			float s_angle = (time.second()-15)*6;
			matrix.rotate( s_angle );
			paint.setWorldMatrix( matrix );
			pts.setPoints(4,0,0,0,0,400,0,0,0);
			if ( second )
				paint.drawPolygon( pts );
			matrix.rotate( -s_angle );

			// quadrante
			for ( int i=0 ; i < 60 ; i++ )
				{
				paint.setWorldMatrix( matrix );
				if ( (i % 5) == 0 )
					paint.drawLine( 450,0, 500,0 );	// draw hour lines
				else	paint.drawPoint( 480,0 );	// draw second lines
				matrix.rotate( 6 );
				}

			if ( tree_d )
				{
				paint.setPen( Fore );
				paint.setBrush( Fore );
				paint.setViewport(0,0,width(),height());
				}
			else	break;
			}
		}
	paint.end();

	// flicker free code by Remi Guyomarch <rguyom@mail.dotcom.fr>
	bitBlt (this, 0,0, &pm); 
}

void Kclock::focusOutEvent( QFocusEvent * )
{
	if ( (ontop) && (!hasFocus()) && (!menu->isVisible()) && (!onmenu) )
		{
		raise();
		repaint();
		}
}

// ***********************************************************************
// |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// >>>>>>>>>>>>>>>>>>>>>>>>> Our trip start here <<<<<<<<<<<<<<<<<<<<<<<<<
// |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// ***********************************************************************
int main( int argc, char **argv )
{
	KApplication app( argc, argv );
	Kclock *clock = new Kclock;
	app.setMainWidget( clock );
	clock->show();
	app.exec();
}
