/*
 *  Name        : kclock.h
 *  Version     : 0.6
 *  Description : A clock with some more...
 *
 *  Copyright (C) 1998 Luca Montecchiani <m.luca@usa.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef ACLOCK_H
#define ACLOCK_H 

#include <qtimer.h>
#include <qcombo.h>
#include <qtabdlg.h>
#include <qpainter.h>
#include <qmsgbox.h>
#include <qwidget.h>
#include <qdatetm.h>
#include <qpopmenu.h> 
#include <qapp.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qlined.h>
#include <qpushbt.h>
#include <qclipbrd.h>

extern "C" {
#include <mediatool.h>
}

#include <kapp.h> 
#include <kfontdialog.h>
#include <kcolordlg.h>
#include <klocale.h>
#include <kaudio.h> 

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define VER	6
#define ktr klocale->translate

/* XPM */
#include "xpm/kclock.xpm"

class Kclock : public QWidget
{
	Q_OBJECT
public:
	Kclock( QWidget *parent=0, const char *name=0 );

protected:
	virtual void	paintEvent( QPaintEvent *event );
	virtual void	mousePressEvent( QMouseEvent *event );
	virtual void	mouseMoveEvent( QMouseEvent *event );
	virtual void	focusOutEvent( QFocusEvent *event );
	virtual void 	resizeEvent( QResizeEvent *event ); 

private slots:
	void		Timeout();
	void		About();
	void		Hints();
	void		Quit();
	void		Show_second();
	void		Show_date();
	void		Show_ontop();
	void		Show_digital();
	void		Show_tree_d();
	void		Save_config();
	void		Set_time();
	void		Help();
	void		Beep();
	void		Show_border();
	void		selectFont();
	void		ResetFont();
	void		selectShad();
	void		selectFore();
	void		selectBack();
	void		resetCol();
	void		Initialize();
	void		Speak();

private:
	QTime		time;
	QString		Buf;
	QPopupMenu 	*menu;
	QPopupMenu 	*menu1;
	QPopupMenu 	*menu2;
	QFont 		font;
	QColor		Back;
	QColor		Fore;
	QColor		Shad;

	bool		second;
	bool		digital;
	bool		date;
	bool		ontop;
	bool		tree_d;
	bool		beep;
	bool		speak;
	bool		onmenu;
	bool		border;
	bool		sound;
	int		ver;

	int		secondID;
	int		digitalID;
	int		dateID;
	int		ontopID;
	int		beepID;
	int		tree_dID;
	int		borderID;
	int		speakID;
};

#endif
